import matplotlib.pyplot as plt
import numpy as np
import time
import sys
import statistics  #pour faire la moyenne des temps d'insertion pour chaque facteur de charge



###### PARTIE 2 ######

def experiment_load_factor(load_factors : list):
	"""
	Étude du facteur de charge
	Parameters
	----------
	load_factors : list
		Une liste de facteurs de charge
	Returns
	-------
	lists
		Les temps d'insertion de clefs
		Les nombres de réallocations de mémoire
		Les tailles de mémoire occupée par le dictionnaire pour chaque facteur de charge
	"""
	# Initialisation
	insertion_times = []
	num_resizes = []
	sizes = []

	for factor in load_factors :
		dictio = {}
		num_resize=0
		last_size = sys.getsizeof(dictio)
		num_elements = int(factor*100)
		tempsecoules = []

		for i in range(num_elements) :
			cle = 'cle'+str(i)
			start_time = time.perf_counter()
			dictio[cle] = i
			end_time = time.perf_counter()
			tempsecoule = end_time - start_time
			tempsecoules.append(tempsecoule)
			current_size = sys.getsizeof(dictio)

			if current_size > last_size :
				num_resize += 1
				last_size = current_size

		size = sys.getsizeof(dictio)
		insertion_time = statistics.mean(tempsecoules)
		insertion_times.append(insertion_time)
		num_resizes.append(num_resize)
		sizes.append(size)

	return insertion_times, num_resizes, sizes

def experiment_longest():
	"""
	Cette fonction renvoie la fréquence des temps d'insertion dans un dictionnaire, sur 10 000 éléments à insérer.
	Returns
	-------
	dict
		Les fréquences des temps d'insertion
	"""
	d = {}
	insertion_times = []

	for i in range(10000):
		key = str(i)
		value = i
		start_time = time.time()
		d[key] = value
		insertion_time = time.time() - start_time
		insertion_times.append(insertion_time)
	frequencies = np.histogram(insertion_times)[0]
	return frequencies

def visualisation(load_factors, insertion_times, num_resizes, sizes, frequencies):
	"""
	Visualisation des résultats
	Parameters
	----------
	load_factors : list
		La liste des facteurs de charge
	insertion_times : list
		La liste des temps d'insertion de clefs
	num_resizes : list
		Le nombre de changements de taille du dictionnaire
	sizes : list
		Les tailles du dictionnaire
	frequencies : dict
		Les fréquences des temps d'insertion
	Returns
	-------
		4 graphes
	"""
	num_elements = []
	for factor in load_factors :
		num_elements.append(factor*100)
	# Temps d'insertion en fonction du facteur de charge
	plt.figure()
	plt.plot(load_factors, insertion_times, marker='o', linestyle='-')
	plt.yscale("log")
	plt.xlabel("Facteur de charge")
	plt.ylabel("Temps d'insertion (secondes)")
	plt.title("Temps d'insertion en fonction du facteur de charge")
	plt.savefig("temps_d_insertion.png")
	plt.close()
	# Nombre de réallocations de mémoire en fonction du facteur de charge
	plt.figure()
	plt.plot(load_factors, num_resizes, color='orange', marker='o', linestyle='-')
	plt.xlabel("Facteur de charge")
	plt.ylabel("Nombre de réallocations de mémoire")
	plt.title("Nombre de réallocations de mémoire en fonction du facteur de charge")
	plt.savefig("nombre_reallocations.png")
	plt.close()
	# Taille de mémoire occupée en fonction du nombre d'éléments
	plt.figure()
	plt.plot(num_elements, sizes, color='green', marker='o', linestyle='-')
	plt.xlabel("Nombre d'éléments")
	plt.ylabel("Taille de mémoire occupée (octets)")
	plt.title("Taille de mémoire occupée en fonction du nombre d'éléments")
	plt.xticks(rotation=45)
	plt.savefig("taille_memoire.png")
	plt.close()
	# Deuxième étude
	f = list(frequencies)
	plt.figure(figsize=(10, 6))
	plt.bar(range(len(f)), f)
	plt.xlabel('Temps d\'insertion (s)')
	plt.ylabel('Fréquence')
	plt.title('Histogramme des fréquences des temps d\'insertions')
	plt.yscale('log')
	xticks = np.logspace(-6, 1, 3)
	xtick_labels = [f'{x:.1e}' for x in xticks]
	plt.xticks(xticks, xtick_labels)
	plt.savefig('deuxieme_etude.png')

load_factors = [0.01, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
insertion_times, num_resizes, sizes = experiment_load_factor(load_factors)
frequencies = experiment_longest()
visualisation(load_factors, insertion_times, num_resizes, sizes, frequencies)
